C--------------------------------------------------------------------
C     SOLVE EQUATION (10) IN REFERENCE.
C     calculate the derivatives dC/drho for F-, A- and G-constraints
C--------------------------------------------------------------------
C
C*   IDX(1..3,1..nref) contains the hkl-indices of ALL reflections
C*   FOBS(1..2,1..nrefF) contains the A and B of F-constraints
C*     SIGF(1..nrefF) contains the sigmas(|F|) of the F-constraints
C*   AOBS(1..2,1..nrefA) contains the A and |B| of A-constraints 
C*     SIGA(1..2,1..nrefA) contains the sigma(A) and sigma(|B|) (A-constraints)
C*     COSPSI(1..nrefA),SINPSI(1..nrefA) contain the phase angles of the
C*       structure factor of the sub-structure of the anomalous scatterer
C*       for the A-constraints reflections
C*   GOBS(1..nrefG) contains the mean |F| of G-constraints
C*     SIGG(1..nrefG) contains the sigma of Gobs of group j   
C*     NGRP(1..nrefG) contains the number of reflections in group j   
C*     PMUL(1..nrefG+..) contains the multiplicity of reflections in groups 
C*     PMULS(1..nrefG) contains the sum of the multiplicities of group j    
C*   nrefF,nrefA,nrefG contain the numbers of constraints: F, A, G
C*   nref=total number of reflections>=nrefF+nrefA+nrefG
C*   FCAL(1..2,1..nref) contains the calculated A and B of ALL constraints
C*   GCAL(1..nrefG) contains the calculated |G| of the G-constraints
C
      SUBROUTINE CMAXENT(ngsw,mref,nrefF,nrefA,nrefG,masym,nasym,rtasym
     $  ,FOBS,FCAL,PROF,SIGF,NGRP,GOBS,GCAL,SIGG,AOBS,SIGA,
     $   RMIC,lambda,COSPSI,SINPSI,ASCAL,BSCAL
     $  ,RMIS
     $  ,PMUL,PMULS,RDEO,RDEN,RHELP1,RHELP2,RHELP3)
C
      parameter (rminden=1.e-4)		!min. density (e/A^3)
      integer*4 masym,nasym,ia
      integer ngsw,mref,nrefF,nrefA,nrefG,NGRP(mref/2)
      real*8 derF,derA,derG,rtasym,lambda
C     *** density arrays ***
      real*8 RDEO(masym),RDEN(masym)
C     *** structure factor lists ***
      real*8 FOBS(2,mref),FCAL(2,mref),SIGF(mref),PROF(mref),
     $    GOBS(mref/2),GCAL(mref/2),SIGG(mref/2),
     $      PMUL(mref/2),PMULS(mref/2),
     $    AOBS(2,mref),SIGA(2,mref),COSPSI(mref),SINPSI(mref),
     $      ASCAL(mref),BSCAL(mref)
C     *** cosine and sinus-tables ***
      real*8 RMIC(mref,masym)
     $      ,RMIS(mref,masym)
C     *** auxilary arrays ***
      real*8 RHELP1(mref),RHELP2(mref),RHELP3(mref),sum1
      integer iref,jref,nrefFA
C
C     *** precalculations ***
C
      nrefFA=nrefF+nrefA
C
C     *** 1. `normal' F-constraints: A,B are given
C
      do 1 iref=1,nrefF
	 RHELP1(iref)= (FCAL(1,iref)*PROF(iref)-FOBS(1,iref))
     $	           /(SIGF(iref)*rtasym)
         RHELP2(iref)= (FCAL(2,iref)*PROF(iref)-FOBS(2,iref))
     $             /(SIGF(iref)*rtasym)
   1  continue
C
C     *** 2. this part only, if A-constraints: A,|B| are given
      if ((nrefA.gt.0)) then
        do 2 iref=1,nrefA
          jref=nrefF+iref
          ASCAL(iref)=FCAL(1,jref)*PROF(jref)*COSPSI(iref)
     $              + FCAL(2,jref)*PROF(jref)*SINPSI(iref) 
          BSCAL(iref)=dsqrt((-FCAL(1,jref)*PROF(jref)*SINPSI(iref)
     $              + FCAL(2,jref)*PROF(jref)*COSPSI(iref) 
     $                    )**2 )
          RHELP1(jref)= (ASCAL(iref)-AOBS(1,iref))
     $	                /(SIGA(1,iref)*rtasym)
          if (ngsw.eq.0) then
          RHELP2(jref)= (BSCAL(iref)-AOBS(2,iref)) 
     $                  /(BSCAL(iref)*SIGA(2,iref)*rtasym)
          endif
   2    continue
      endif
C
C     *** 3. this part only, if G-constraints:  Gobs are given = sqrt(
C     ***    mean |F|^2) of a group of reflections
      IF (ngsw.EQ.0) THEN
        jref=nrefFA+1
        DO 3 iref=1,nrefG
          DO 4 igrp=1,NGRP(iref)
   	  RHELP1(jref)= PMUL(jref-nrefFA)*FCAL(1,jref)*PROF(jref)
          RHELP2(jref)= PMUL(jref-nrefFA)*FCAL(2,jref)*PROF(jref)
          jref=jref+1
 4        continue
          RHELP3(iref)= (GCAL(iref)-GOBS(iref))/
     $           (PMULS(iref)*GCAL(iref)*SIGG(iref)*rtasym)
 3      continue
      ENDIF
C     *** end precalculations ***
C
C     *** now calculate derivatives for each point in asymmetric unit ***
C
      derF=0.0d0
      derA=0.0d0
      derG=0.0d0
      DO 99 ia=1,nasym
         derF=0.0d0
C
C*** DERIVATIVE FOR F-TYPE CONSTRAINT ***
C
         DO 20 iref=1,nrefF
           derF=derF + RHELP1(iref)*RMIC(iref,ia)
     $               + RHELP2(iref)*RMIS(iref,ia)
   20    CONTINUE
         if (nrefF.gt.0) derF=derF/nrefF
C
C*** DERIVATIVE FOR A-TYPE CONSTRAINT ***
C        *** add derivative of |B| only, if not first cycle with a flat
C        *** prior density
C
         IF ((nrefA.gt.0)) THEN
           derA=0.0d0
           DO 25 iref=1,nrefA
             jref=nrefF+iref
             derA=derA + RHELP1(jref)*(
     $         COSPSI(iref)*RMIC(jref,ia)
     $         + SINPSI(iref)*RMIS(jref,ia)
     $         )
             if (ngsw.eq.0) derA=derA+ RHELP2(jref)*(
     $         FCAL(1,jref)*(SINPSI(iref)**2*RMIC(jref,ia)-
     $                       SINPSI(iref)*COSPSI(iref)*RMIS(jref,ia))
     $        +FCAL(2,jref)*(COSPSI(iref)**2*RMIS(jref,ia)-
     $                       SINPSI(iref)*COSPSI(iref)*RMIC(jref,ia)) )
   25      CONTINUE
           derA=derA/nrefA
         END IF
C
C*** DERIVATIVE FOR G-TYPE CONSTRAINT ***
C
         IF (NGSW.EQ.0) THEN
           derG=0.0d0
           jref=nrefFA+1
           DO 40 iref=1,nrefG
             sum1=0.0
             DO 50 igrp=1,NGRP(iref)
               sum1=sum1+RHELP1(jref)*RMIC(jref,ia)
     $                  +RHELP2(jref)*RMIS(jref,ia)
               jref=jref+1
   50        CONTINUE
             derG=derG+sum1*RHELP3(iref)
   40      CONTINUE
           if (nrefG.gt.0) derG=derG/nrefG
         END IF
C
C*** SOLVE EQ.(10) IN REFERENCE. ***
C
         RDEN(ia)=RDEO(ia)*DEXP(-lambda*(derF+derA+derG))
         if (rden(ia).lt.rminden) rden(ia)=rminden
C
   99 CONTINUE
C
      RETURN
      END
