C========================================================================
C
C     smeedcab.f -- for NON-centrosymmetric samples; DOUBLE precision
C                -- with A, |B| option as 'A-constrains' added
C
C     MAXIMUM ENTROPY ELECTRON DENSITY ANALYSIS PROGRAM
C     FOR X-RAY POWDER DIFFRACTION DATA
C
C     REFERENCE : J.APPL.CRYST. (1990) 23, 526-534
C                 J.Appl.Cryst. (1993) 26, 453-457
C       K. Burger, Dissertation (thesis), Universitaet Tuebingen 1997.
C         (published by Shaker, Aachen, Germany)
C       Burger & Prandl, A new type of constraint in the
C         Maximum-Entropy method using ambiguous phase information from 
C         anomalous scattering powder data, submitted to Acta. Cryst.  1998.
C
C     COPYRIGHT BY SHINTARO KUMAZAWA ( NAGOYA UNIVERSITY )
C         and      K. Burger, Universitaet Tuebingen
C     ALL RIGHT RESERVED.
C
C     - UPDATE LABEL -
C
C     VERSION 1.00  1990-04-24  BY SHINTARO KUMAZAWA
C      ORIGINAL PROGRAM, MEMTARO.FORT.
C     VERSION 1.10  1990-11-08  BY YOSHIKI KUBOTA
C      ADD PREFERED ORIENTATION CORRECTION VERSION.
C     VERSION 2.00  1991-10-16  BY SHINTARO KUMAZAWA
C      START MEED SYSTEM PROJECT.
C      CHANGE SYMMETRY TREATMENT AND SOME CALCULATION ALGORITHM.
C     VERSION 2.12  1991-10-25  BY SHINTARO.KUMAZAWA
C      BUG FIX & OPTIMIZE SOME CALCULATION ALGORITHM.
C     VERSION 3.00  1991-11-15  BY SHINTARO KUMAZAWA
C      MODIFIED BY USING OF MAU METHOD.
C     Data input now read free-format. Better handling of divergence errors 
C      - 12.1995 k.b.
C     Data output format changed  - K. Burger 3.1996
C     MaxEnt-routine optimized for velocity.
C      New COMMON variables: rhelp1,2,4,5,6(mref) for MAXENT-SUBROUTINE.
C      Changes within loop over maxent cycles.
C      Memorize FCAL and GCAL from last successful cycle in case of
C      divergence - K. Burger 3.96 
C     lambda is varied automatically: in case of divergence lambda=lambda*0.5
C      if cycle was successful: lambda is increased a bit.
C      if the constraint value varies too quickly, lambda is reduced.
C      Bug in MKASYM routine fixed: wrong results in the case of
C      different pixel resolution: NPIX(1)<>NPIX(2 or 3).
C      Translation vectors ITRS and ISUB now real numbers: RTRS and RSUB.
C     Program split into modules; no COMMON blocks any more. (k.b. 4.1996)
C     With A,|B|-constraints added -- k.b. December 1996
C     Bug in constraint-value calculation in first cycle fixed,
C       added `deviations' in result-list. k.b. Sept. 1997.
C
C     - CAUTION !! -
C
C     (1) IF YOU TRY A SAMPLE HAVING NO CENTRE OF SYMMETRY, YOU MUST
C         REMOVE THE COMMENT MARK, '    '.
C         Also, calls to subroutines MKASYM, STRCTF,STRCTG,CALCON,MAXENT
C         have to be substituted to their corresponding ones CMKASYM,CSTRCF,
C         CSTRCG,CCALCON,CMAXENT
C           
C     (2) YOU MAY NOT USE AN OLD FORMAT MEM DATA SET.
C
C     - UNIT NUMBER OF DATA FILES -
C
C     10 : MEED DATA SET ( THIS IS POWLS6.EXE COMPATIBLE FORMAT. )
C     20 : DENSITY DATA SET ( THIS IS READABLE BY MECONT.EXE. )
C     30 : ANALYSIS DOCUMENT DATA SET
C     40 : SPACE GROUP DATA BASE 'SPGRA.DAT', OFFERED BY FUJIO IZUMI.
C          (needed in subroutine rdspg.f)
C
C====================================================================
C
C
      PROGRAM smeedcab
C
C       PARAMETERS:
C       mref = max. number of reflections
C       masym = max. number of pixels in asymmetric unit of cell
C       mpix = max. number of pixels along one cell axis (resolution)
C       mhist = max. number of changes in lambda-parameter done by the user
C       alpha = Final value of constraint
C       mautosave = number of cycles, after which an automatic savig of the 
C               current density is done.
C
      IMPLICIT none
C
      integer*4 nasym,masym,ia
      integer mhist,mref,mpix,mautosave
      real*8 alpha
C
      PARAMETER ( mref=160,masym=6000,mpix=44)
      PARAMETER ( mhist=90, mautosave=200 )
      PARAMETER ( alpha=1.0 )
C
C     *** VARIABLES ***
C
C*   IDX(1..3,1..nref) contains the hkl-indices of ALL reflections
C*   FOBS(1..2,1..nrefF) contains the A and B of F-constraints
C*     SIGF(1..nrefF) contains the sigmas(|F|) of the F-constraints
C*   AOBS(1..2,1..nrefA) contains the A and |B| of A-constraints 
C*     SIGA(1..2,1..nrefA) contains the sigma(A) and sigma(|B|) (A-constraints)
C*     COSPSI(1..nrefA),SINPSI(1..nrefA) contain the phase angles of the
C*       structure factor of the sub-structure of the anomalous scatterer
C*       for the A-constraints reflections
C*   GOBS(1..nrefG) contains the mean |F| of G-constraints
C*     SIGG(1..nrefG) contains the sigma of Gobs of group j   
C*     NGRP(1..nrefG) contains the number of reflections in group j   
C*     PMUL(1..nrefG+..) contains the multiplicity of reflections in groups 
C*     PMULS(1..nrefG) contains the sum of the multiplicities of group j    
C*   nrefF,nrefA,nrefG contain the numbers of constraints: F, A, G
C*   nref=total number of reflections
C*   FCAL(1..2,1..nref) contains the calculated A and B of ALL reflections
C*   GCAL(1..nrefG) contains the calculated |G| of the G-constraints
C
C     *** variables connected with the scattering-density values in the 
C     asymmetric unit of the cell:
      real*8 RDEO(masym),RDEN(masym),RASYM(masym) 
C
C     *** variables connected with the coordinates of the pixels:
      integer NPIX(3),IASYM(3,masym)
C
C     *** variables connected with the structure factors given as input:
C     *** FOBS: unique reflections, GOBS: groups of non-unique refl.
      real*8 FOBS(2,mref),FCAL(2,mref),GOBS(mref/2),GCAL(mref/2),
     $    AOBS(2,mref),SIGA(2,mref),SIGF(mref),SIGG(mref/2)
     $    ,PROF(mref),COSPSI(mref),SINPSI(mref)
      real*8 tnref
      integer IDX(3,mref),NGRP(mref/2),nref,nrefF,nrefG,nrefA
C
C     *** cell-constants and others:
      real*8 CELL(6),pop,rtasym
      integer jph,jpk,jpl
      real*8 pi,rad,ca,cb,cg,sa,sb,sg,
     $   da,db,dg,dv,sv,as,bs,cs,ga,gb,gg,rh,rk,rl,av,
     $   rdint,totale,ff1,ff2
C
C     *** variables connected with the entropy-functions:
      real*8 rfact,wrfact,ents,entj,qfunc,conF,conA,conG,connew,
     $  delta,lambda
C
C     *** arrays to memorize lquantities of last successful cycle 
      real*8 FCALO(2,mref),GCALO(mref/2),ACALO(2,mref),
     $  conFold,deltaold,rfacold,wrfacold,conold,conGold,conAold
C
C     *** variables connected with symmetry operators:
      integer IROT(3,3,48),nsym,ncen,nsub
      real*8 RTRS(3,48),RSUB(3,4)
C
C     *** tables of cos- and sin-values for each reflection and pixel:
      real*8 RMIC(mref,masym)
      real*8 RMIS(mref,masym)
C
C     *** variables connected with lambda-history list:
      integer IHIST(MHIST),nhist
      real*8 FHIST(3,MHIST)
C
C     *** variables connected with multiplicities of reflections given as groups
C     of overlayd reflections (G_obs, G-constraints):
      real*8 PMUL(mref/2),PMULS(mref/2)
C
C     *** auxilary arrays for maxent subroutine
      real*8 RHELP1(mref),RHELP2(mref),RHELP3(mref),ASCAL(mref),
     $       BSCAL(mref)
C
C     *** auxilary array for mkasym subroutine
      integer IDE(0:mpix,0:mpix,0:mpix)
C
C     *** other variables:
      CHARACTER title*200,filmem*50,filden*50,fildoc*50,
     $          aspg*10
      character*10 APRIOR(2)
      character*6 AGSW(2)
      integer iprior,igsw,nyes,iptyp,nsg,nset,
     $  ierr,itcycl,ngsw,iref,nn,i,j,k,
     $  ncycl,iii,icycl,ibadruns,igoodcyc
C     integer ITODAY(4)
      DATA (APRIOR(J),J=1,2)/'FLAT.     ','FILE DATA. '/
      DATA (AGSW(J),J=1,2)/'F & AG','   F  '/
C
C
 1090 format(2(3(I3,1X),G18.13,1x))  ! for display of asym. unit pixels
 2010 FORMAT(BN,A50)                 ! for filenames
 2020 FORMAT(BN,I5)		     ! for input of yes/no=0/1
 3000 FORMAT(4(/' '))
 3010 FORMAT(///'   MMM            MMM   EEEEEEEEEEEEEEE   EEEEEEEEEE',
     $       'EEEEE   DDDDDDDDDD'/'   MMMMMM      MMMMMM   EEEEEEEEEE',
     $       'EEEEE   EEEEEEEEEEEEEEE   DDDDDDDDDDDD'/'   MMM MMM    ',
     $       'MMM MMM   EEE               EEE               DDD      ',
     $       '  DDD'/'   MMM  MMM  MMM  MMM   EEE               EEE  ',
     $       '             DDD         DDD'/'   MMM   MMMMMM   MMM   ',
     $       'EEEEEEEEEEEEEEE   EEEEEEEEEEEEEEE   DDD          DDD'
     $       /'   MMM     MM     MMM   EEEEEEEEEEEEEEE   EEEEEEEEEEEE',
     $       'EEE   DDD          DDD'/'   MMM            MMM   EEE   ',
     $       '            EEE               DDD         DDD'/'   MMM ',
     $       '           MMM   EEE               EEE               DD',
     $       'D        DDD'/'   MMM            MMM   EEEEEEEEEEEEEEE ',
     $       '  EEEEEEEEEEEEEEE   DDDDDDDDDDDD'/'   MMM            MM',
     $       'M   EEEEEEEEEEEEEEE   EEEEEEEEEEEEEEE   DDDDDDDDDD'
     $       //'   MAXENT ELECTRON DENSITY ANALYSIS PROGRAM FOR X-RA',
     $       'Y POWDER DIFFRACTION DATA'//'   REFERENCE : J.APPL.CRYS',
     $       'T. (1990) 23, 526-534;  (1993) 26, 453-457.',/,15x,
     $       'K. Burger, Dissertation (thesis), Univ. Tuebingen 1997.',
     $      /'   COPYRIGHT BY SHINTARO KUMAZ',
     $       'AWA (C) 1991, K. Burger 1997,  ALL RIGHT RESERVED.',
     $      //' >>> for CENTROSYMMETRIC SAMPLES <<<',
     $       /' >>> or NON-CENTROSYMMETRIC SAMPLES <<<',
     $      / ' with the new A-constraints implemented!',
     $      //' SMEEDCAB Version of Dr. K. Burger,',/,' Institut fuer',
     $        ' Kristallographie, Universitaet Tuebingen',
     $       /' Charlottenstr. 33, D-72070 Tuebingen, Germany, ',
     $        ' (March a.d. 1998)' )
 3020 FORMAT(/' *********   INITIAL STATE OF MEED ANALYSIS   ******',
     $       '*************************',
     $       /A80, /A80,/ A40/
     $        /' MEED DATA FILE NAME     = ',A50
     $        /' DENSITY DATA FILE NAME  = ',A50
     $        /' DOCUMENT DATA FILE NAME = ',A50
     $       //' CELL CONSTANT = ',5(F8.3,','),F8.3
     $        /' SPACE GROUP   = ',A10,'  ,  TOTAL CHARGE = ',F10.3
     $        /' P.O. CORRECT  = ',F10.5,'  ,  TYPE = ',I2,
     $         '  ,  VECTOR = ',2(I3,','),I3
     $        /' NUMBER OF PIXELS = ',I7,' / ',I7,'   (',2(I3,','),
     $         I3,' )'
     $       //' REFLECTIONS OF CONSTRAINT FUNCTION F-TYPE =',I4,
     $         ', A-TYPE =',I4,', G-TYPE =',I4
     $        /' INITIAL DENSITY DISTRIBUTION IS ',A
     $        /' THIS SESSION IS ANALYZED BY USING ',A,' CONSTRAINT.'
     $        /' TRIAL LAGRANGE MULTIPLIER  lambda = ',F10.7,'.')
 3030 FORMAT( ' con=',F15.2,' conF=',F13.2,' conA=',F13.2,
     $           ' conG=',F13.2,
     $        /'delt=',F18.5,'    R=',F15.4,'   wR=',F15.4)
 3040 FORMAT(  'ents=',F18.5,' entj=',F15.4,' lagr=',F15.4)
 3060 FORMAT(' CYCLE ',I5)
 3130 FORMAT(/' *********   WAIT A MOMENT ! THE PHASE MATRIX ',
     $       'TABLE IS CALCULATED. ******')
 3500 FORMAT(/ ' INPUT MEED FILE NAME     = ',$)
 3510 FORMAT(  ' INPUT DENSITY FILE NAME  = ',$)
 3520 FORMAT(  ' INPUT DOCUMENT FILE NAME = ',$)
 3530 FORMAT(  ' SELECT INITIAL DENSITY DATA TYPE.  0=FLAT, 1=',
     $       'READ IN FILE DATA : ',$)
 3540 FORMAT(  ' DO YOU USE OTHER CONSTRAINTS (G or A) THAN USUAL (F)'
     $       , ' IN THE CALCULATION ?',/,' (0=YES, 1=NO) : ',$)
 3550 FORMAT(  ' ARE YOU SURE ?  (0=YES, 1=NO) : ',$)
 3560 FORMAT(/' INPUT NCYCL. ( >0:ITERATION CYCLES , <0:CHANGE LAMD',
     $       'A , =0:EXIT )  : ',$)
C
C*** INPUT FILENAMES AND INITIAL CONDITIONS. ***
C
      WRITE(6,3010)
  201 WRITE(6,3500)
      READ(5,2010) filmem
      WRITE(6,3510)
      READ(5,2010) filden
      WRITE(6,3520)
  203 READ(5,2010) fildoc
      OPEN(30,FILE=fildoc,FORM='FORMATTED',STATUS='UNKNOWN',
     $     ERR=203,ACCESS='SEQUENTIAL')
      write(30,3010)
      WRITE(6,3530)
      READ(5,2020) iprior
      WRITE(6,3540)
      READ(5,2020) igsw
      WRITE(6,3550)
      READ(5,2020) nyes
      IF (nyes.NE.0) GO TO 201
C
C*** READ MEED DATA AND MAKE SOME CONSTANTS. ***
C
      ierr=0
206   CALL READMEM(filmem,title,CELL,pop,iptyp,jph,jpk,jpl,
     $  nsg,nset,NPIX,mpix,totale,lambda,mref,nref,nrefF,nrefG,
     $  nrefA,IDX,FOBS,SIGF,GOBS,SIGG,NGRP,PMUL,PMULS,
     $  AOBS,SIGA,COSPSI,SINPSI,ierr,igsw)
      if (ierr.ne.0) then 
        write(*,*) 'ERROR: MEM-file could not be opened: ',filmem
        write(*,*)'Please input a new name:'
        read(*,'(a)') filmem
        goto 206
      endif
C
      CALL RDSPG(nsg,nset,aspg,nsym,ncen,nsub,
     $  IROT,RTRS,RSUB,CELL)
C
C     *** display of symmetry info ***
C     CALL dspsym(nsg,nset,aspg,nsym,ncen,nsub,
C    $  IROT,RTRS,RSUB,CELL)
C
      CALL PRECAL(jph,jpk,jpl,pi,rad,ca,cb,cg,sa,sb,sg,
     $   da,db,dg,dv,sv,as,bs,cs,ga,gb,gg,rh,rk,rl,av,
     $   rdint,totale,ff1,ff2,CELL,NPIX)

      CALL POCOR(iptyp,pop,PROF,nref,mref,IDX,
     $	               as,bs,cs,ga,gb,gg,rh,rk,rl,pi)
C
C*** INITIALIZE DATA & OUTPUT INITIAL STATE. ***
C
  210 WRITE(6,3130)
      IF (iprior.EQ.0) THEN
         CALL CMKASYM(0,masym,nasym,mpix,NPIX,
     $     IROT,RTRS,RSUB,nsym,ncen,nsub,
     $     RDEN,rtasym,RASYM,rdint, IASYM,RMIC,
     $     RMIS,
     $     IDX,mref,nref,IDE,pi)
      ELSE
	 CALL READDEN(filden,masym,nasym,IASYM,RDEN)
         CALL CMKASYM(1,masym,nasym,mpix,NPIX,
     $     IROT,RTRS,RSUB,nsym,ncen,nsub,
     $     RDEN,rtasym,RASYM,rdint, IASYM,RMIC,
     $     RMIS,
     $     IDX,mref,nref,IDE,pi)
      END IF
C     print *,' nsym,nsub,ncen'
C     print *,nsym,nsub,ncen,(npix(i),i=1,3)
C     write(*,'(12i5)') (((irot(i,j,k),I=1,3),j=1,3),
C    $  (nint(npix(i)*rtrs(i,k)),i=1,3),
C    $    k=1,nsym)
C     write(*,'(3i5)') ((nint(npix(i)*rsub(i,k)),i=1,3),k=1,nsub)
C     read(*,*) idummi
C
C     ***** do precalculations for maxent *****
C
      nhist=0
      itcycl=1
      delta=-1.0D-9
C     *** if no Gobs are given: ngsw=1
C     *** if Gobs are given: in the first cycle, ngsw=-1 and the Gobs are 
C     *** not considered in the maxent-process. This is because Gcal is 
C     *** zero for flat starting-density, and there would result a `divided
C     *** by zero' error in the MAXENT routine.
C     *** After the first cycle, ngsw is set 0 in this case.
      ngsw=1
      if (nrefG.gt.0) then
        if (iprior.eq.0) then
C         *** Gobs given and flat starting density ***
          ngsw=-1
        else 
C         *** Gobs and non-flat starting density from file ***
          ngsw=0
        endif
      endif
C     *** rden --> rdeo ***
      CALL RENEW(masym,nasym,RDEO,RDEN)
C     *** calculate and memorize F_calc ***
      CALL CSTRCTF(masym,nasym,RDEN,RMIC
     $   ,RMIS
     $   ,RASYM,mref,nref,FCAL,ff1)
C     *** memorize FCAL
      DO 42 iref=1,nref
	 FCALO(1,iref)=FCAL(1,iref)
	 FCALO(2,iref)=FCAL(2,iref)
42    CONTINUE
      nn=nrefF+nrefA
      IF (ngsw.le.0) THEN
         nn=nn+nrefG
      ENDIF
      tnref=DFLOAT(nn)
      IF (ngsw.le.0) THEN
         CALL CSTRCTG( mref,nrefF,nrefA,nrefG,FCAL,GCAL,
     $      PROF,NGRP,PMUL,PMULS)
C        *** memorize GCAL
         DO 43 iref=1,nrefG
 	   GCALO(iref)=GCAL(iref)
	   GCALO(iref)=GCAL(iref)
43       CONTINUE
      ENDIF
C     *** calculate constraint values ***
      CALL CCALCON(ngsw,conF,conA,conG,wrfact,rfact,connew,
     $   mref,nrefF,nrefA,nrefG,FCAL,PROF,FOBS,SIGF,
     $   AOBS,SIGA,COSPSI,SINPSI,GCAL,GOBS,SIGG )
      conold=connew
C     *** calculate entropy-function values ***
      CALL CALFN(masym,nasym,entj,ents,rtasym,RDEN,RDEO,RASYM,
     $   ff2,qfunc,lambda,connew)
      DO 60 I=1,2
         nn=6
         IF (I.EQ.2) nn=30
         WRITE(nn,3020)
     $   title(1:80),title(81:160),title(161:200),
     $   filmem,filden,fildoc,(CELL(j),j=1,6),aspg,totale,pop,iptyp,
     $   jph,jpk,jpl,nasym,NPIX(1)*NPIX(2)*NPIX(3),(NPIX(j),j=1,3),
     $   nrefF,nrefA,nrefG,APRIOR(iprior+1),AGSW(igsw+1),lambda
         WRITE(nn,3030) connew,conF,conA,conG,delta,rfact,wrfact
         WRITE(nn,3040) ents,entj,qfunc
   60 CONTINUE
C
C     ************************************************************
C     *** MAXENT PROCESS ***
C     ************************************************************
C
C     ***** start of loop over all cycles of calculation *********
C
  220 WRITE(6,3560)
      READ(5,2020,ERR=220) ncycl
      IF (ncycl.LT.-100) then
C       *** start from the beginning ***
        GO TO 210
      elseif (ncycl.lt.0) then
C       *** input different lambda/alpha ***
        CALL HISTRY(itcycl,delta,alpha,lambda,nhist,
     $                  mhist,FHIST,IHIST)
        goto 220
      elseif (ncycl.eq.0) then
C       *** stop calculations ***
        print *,
     $    'Do you really want to stop the calculation ? (0=yes)'
        iii=0
        read(5,2020) iii
        if (iii.ne.0) goto 220
C       *** stop and save density in file: ***
        goto 260 
      elseif (ncycl.gt. 0) then
C       *** normal case: positive number of cycles given ***
	icycl=1
        ibadruns=0
        igoodcyc=0
      endif
C     -----------------------------------------------
C     ***** start of loop over NCYCL cycles *********
C     -----------------------------------------------
240   if (ibadruns.gt.13) then
C        *** exit loop: save and display last sensible results ***
	 print *,'-------------------------------------------'
	 print *,'Exit with saving of data: divergence error!'
	 print *,'-------------------------------------------'
	 itcycl=igoodcyc
	 connew=conold
	 conF=conFold
	 conA=conAold
         conG=conGold
	 delta=deltaold
	 rfact=rfacold
	 wrfact=wrfacold
C        *** take old density as result ***
	 do 245,ia=1,nasym
	   rden(ia)=rdeo(ia)
245      continue
         CALL CSTRCTF(masym,nasym,RDEN,RMIC
     $     ,RMIS
     $     ,RASYM,mref,nref,FCAL,ff1)
	 if (ngsw.eq.0) CALL CSTRCTG(mref,nrefF,nrefA,nrefG,
     $                       FCAL,GCAL,PROF,NGRP,PMUL,PMULS)
	 goto 260
      endif
C
      CALL CMAXENT(ngsw,mref,nrefF,nrefA,nrefG,masym,nasym,rtasym
     $  ,FOBS,FCAL,PROF,SIGF,NGRP,GOBS,GCAL,SIGG,AOBS,SIGA,
     $   RMIC,lambda,COSPSI,SINPSI,ASCAL,BSCAL
     $  ,RMIS
     $  ,PMUL,PMULS,RDEO,RDEN,RHELP1,RHELP2,RHELP3)
C
      CALL NORMAL(masym,nasym,RDEN,RASYM,ff2,rtasym)
      CALL CSTRCTF(masym,nasym,RDEN,RMIC
     $  ,RMIS
     $  ,RASYM,mref,nref,FCAL,ff1)
      if (nrefG.gt.0) CALL CSTRCTG(mref,nrefF,nrefA,nrefG,FCAL,GCAL,
     $                            PROF,NGRP,PMUL,PMULS)
      CALL CCALCON(ngsw,conF,conA,conG,wrfact,rfact,connew,
     $   mref,nrefF,nrefA,nrefG,FCAL,PROF,FOBS,SIGF,
     $   AOBS,SIGA,COSPSI,SINPSI,GCAL,GOBS,SIGG )
      delta=connew-conold
C
      WRITE(6,3060) itcycl
      WRITE(6,3030) connew,conF,conA,conG,delta,rfact,wrfact
      IF (delta.GE.0.0 .or. (DABS(delta/conold).gt.0.1) ) then
C        *** divergence error: choose new lambda and restart calc. ***
C        *** or: at start of calculation too large step in contstraint
	 ibadruns=ibadruns+1
	 lambda=lambda/2
         nhist=nhist+1
	 if (nhist.gt.mhist) nhist=mhist
         FHIST(1,nhist)=lambda*2
         FHIST(2,nhist)=lambda
         FHIST(3,nhist)=connew
         IHIST(nhist)=itcycl
         write(*,'(a,i3,a,f13.8)') 'Bad cycles number:',ibadruns,
     $    '   New lambda=',lambda
C        *** call back F_cal, G_cal of last successful cycle ***
C        *** nref=nrefF + (No. of reflections in groups) ***
         DO 242 iref=1,nref
  	   FCAL(1,iref)=FCALO(1,iref)
	   FCAL(2,iref)=FCALO(2,iref)
242      CONTINUE
         IF (ngsw.le.0) THEN
           DO 243 iref=1,nrefG
             GCAL(iref)=GCALO(iref)
	     GCAL(iref)=GCALO(iref)
243        CONTINUE
         ENDIF
C        *** restart calculation with changed lambda ***
         GO TO 240
      ENDIF 
C     *** this cyle was successful: delta < 0 ***
C     *** new density rden --> rdeo ***
      CALL RENEW(masym,nasym,rdeo,rden)
C     *** increase lambda a bit ***
      lambda=lambda*1.02
C     *** memorize a lot of this cycle ***
      ibadruns=0
      igoodcyc=itcycl
      conFold=conF
      conAold=conA
      conGold=conG
      deltaold=delta
      rfacold=rfact
      wrfacold=wrfact
      DO 252 iref=1,nref
         FCALO(1,iref)=FCAL(1,iref)
	 FCALO(2,iref)=FCAL(2,iref)
252   CONTINUE
      IF (ngsw.le.0) THEN
         DO 253 iref=1,nrefG
 	   GCALO(iref)=GCAL(iref)
	   GCALO(iref)=GCAL(iref)
253      CONTINUE
      ENDIF
C     *** increase cycle counters ***
      itcycl=itcycl+1
      icycl=icycl+1
C     *** save density data after every xxth cycle ***
      if (mod(itcycl-1,mautosave).eq.0) then
         print *,'Automatically saving density data ...'
	 CALL SAVEDEN(filden,
     $      masym,nasym,title,IASYM,RDEN,CELL,NPIX
     $      ,nsym,ncen,nsub,IROT,RTRS,RSUB )
      endif
C
      IF (connew.LE.alpha) then
C        *** calculation finished ***  
	 GO TO 260
      endif
      conold=connew
      IF (ngsw.EQ.-1) ngsw=0
      IF (icycl.LE.ncycl) GO TO 240
C
C     ***** end of loop over NCYCL cycles *********
C
      GO TO 220
C
C     ==========================================================
C     ***** end of loop over all cycles of calculation *********
C     ==========================================================
C
C     **********  DISPLAY & OUTPUT ANALYSIS RESULT.  ***********
C
260   write(6,3000)
      itcycl=itcycl-1
C
C*** SAVE DENSITY DATA. ***
C
      CALL SAVEDEN(filden,masym,nasym,title,IASYM,RDEN,CELL,NPIX
     $                ,nsym,ncen,nsub,IROT,RTRS,RSUB )
C
C*** CALCULATE ENTROPY-FUNCTIONS
      CALL CALFN(masym,nasym,entj,ents,rtasym,RDEN,RDEO,RASYM,
     $   ff2,qfunc,lambda,connew)
C
C*** DISPLAY INTO STANDARD OUTPUT ***
      CALL dspresul(6,
     $  connew,conF,conA,conG,delta,rfact,wrfact,ents,entj,qfunc,
     $  IHIST,FHIST,mhist,nhist,iptyp,itcycl,COSPSI,SINPSI,
     $  mref,nrefF,nrefA,nrefG,FOBS,SIGF,FCAL,PROF,AOBS,SIGA,
     $  NGRP,GOBS,SIGG,GCAL,IDX)
C
C*** DISPLAY INTO DOCUMENT FILE ***
      CALL DSPRESUL(30,
     $  connew,conF,conA,conG,delta,rfact,wrfact,ents,entj,qfunc,
     $  IHIST,FHIST,mhist,nhist,iptyp,itcycl,COSPSI,SINPSI,
     $  mref,nrefF,nrefA,nrefG,FOBS,SIGF,FCAL,PROF,AOBS,SIGA,
     $  NGRP,GOBS,SIGG,GCAL,IDX)
C
C*** DISPLAY FIRST DENSITY VALUES
      write(*,*) 'First 10 values of density:'
      write(30,*) 'First 10 values of density:'
      do i=1,10
        write(*,1090) (iasym(j,i),j=1,3),RDEN(i)
        write(30,1090) (iasym(j,i),j=1,3),RDEN(i)
      end do
      print *
      CLOSE(30)
C
C*** END OF PROGRAM ***
C
      STOP
      END
