C*** READ MEED DATA AND MAKE SOME CONSTANTS. ***
C*** The Data in the file consist of:
C* - six header lines with cell constants, space-group, pixel-resolution ...
C* - The list of F-constraints: h k l A B sigma(|F|) (phased reflections)
C* - The list of A-constraints: h k l A' |B'| s(A) s(|B|) psi_sigma
C*   (partly phased reflections, psi_sigma is the phase angle of anomalous
C*   atoms sub-structure)
C* - The list of G-constraints: hkl-list + 1 Gobs and s(Gobs) (groups of refl.)
C*** The A-constraints must not be given after the G-constraints
C*   IDX(1..3,1..nref) contains the hkl-indices of ALL reflections
C*   FOBS(1..2,1..nrefF) contains the A and B of F-constraints
C*     SIGF(1..nrefF) contains the ( sigma(|F|) )^2 of the F-constraints
C*   AOBS(1..2,1..nrefA) contains the A and |B| of A-constraints 
C*     SIGA(1..2,1..nrefA) contains the (sigma(A))^2 and (sigma(|B|))^2 
C      of the (A-constraints)
C*     COSPSI(1..nrefA),SINPSI(1..nrefA) contain the phase angles of the
C*       structure factor of the sub-structure of the anomalous scatterer
C*       for the A-constraints reflections
C*   GOBS(mref) contains the mean |F| of G-constraints: GOBS(1..nrefG)
C*     NGRP(1..nrefG) contains the number of reflections in group j   
C*     SIGG(1..nrefG) contains the (sigma)^2 of Gobs of group j   
C*     PMUL(1..nrefG+..) contains the multiplicity of reflections in groups 
C*     PMULS(1..nrefG) contains the sum of the multiplicities of group j    
C*   nrefF,nrefA,nrefG contain the numbers of constraints: F, A, G
C*   nref=total number of reflections
C
      SUBROUTINE READMEM(filmem,title,CELL,pop,iptyp,jph,jpk,jpl,
     $  nsg,nset,NPIX,mpix,totale,ramda,mref,nref,nrefF,nrefG,
     $  nrefA,IDX,FOBS,SIGF,GOBS,SIGG,NGRP,PMUL,PMULS,
     $  AOBS,SIGA,COSPSI,SINPSI,ierr,igsw)
      IMPLICIT INTEGER (I-N)
      IMPLICIT REAL*8 (A-H,O-Z)
      character filmem*50, title*200
      real*8 CELL(6),FOBS(2,mref),SIGF(mref),GOBS(mref/2),
     $     SIGG(mref/2),AOBS(2,mref),SIGA(2,mref),
     $     PMUL(mref/2),PMULS(mref/2),COSPSI(mref),SINPSI(mref)
      integer NPIX(3),IDX(3,mref),NGRP(mref/2),iwhat,i,j
      character line*30,line2*30,word*7
 1000 FORMAT(A)
 1015 FORMAT(F15.7,4I5)
C
C     *** number of phased reflections given with A and B and sigma(|F|):
      nrefF=0
C     *** number of partly phased reflections with A and |B| and s(A), s(|B|):
      nrefA=0
C     *** number of unphased groups of reflections with |G| and sigma(|G|):
      nrefG=0
     
      OPEN(10,FILE=FILMEM,FORM='FORMATTED',STATUS='OLD',
     $     ACCESS='SEQUENTIAL',ERR=201)
      ierr=0
      goto 202
C     *** error ***
201     print *,'file not found:',filmem
        ierr=1
        CLOSE(10)
        return
C
202   READ(10,1000) title
      READ(10,*) (CELL(I),I=1,6)
      READ(10,1015) pop,iptyp,jph,jpk,jpL
      READ(10,*) nsg,nset,(NPIX(I),I=1,3)
      IF (NPIX(1)-1.GT.mpix.OR.NPIX(2)-1.GT.mpix.OR.
     $  NPIX(3)-1.GT.mpix) 
     $  CALL PROEND('DIMENSION OVER FOR IDE - TOO MANY PIXELS.')
      READ(10,*) totale,ramda
      READ(10,*) nrefF
      IF (nrefF.GT.mref)  then
	print *,'Maximal number of reflections:',mref
	print *,'Given number                 :',nrefF
	CALL PROEND('Too many Fobs given.')
      ENDIF
      DO 10 nref=1,nrefF
        READ(10,*)
     $  (IDX(I,nref),I=1,3),(FOBS(I,nref),I=1,2),SIGF(nref)
        SIGF(nref)=SIGF(nref)**2
   10 CONTINUE
      nref=nref-1
      nrefF=nref
C     *** only Fobs should be read ***
      if (igsw.eq.1) goto 30
C
C     *** try to read other constraints-lists: A or G.
C     *** first a number and then a keyword ***
C
25    READ(10,'(A)',END=30)  line
      read(line,*) n
      if (n.le.0) goto 30
      j = index(line,'-')
      if (j.eq.0) then
        word='G-const'
      else 
        word=line(j-1:j+5)
      endif
     
26    continue
      if (word(1:7).eq.'A-CONST' .or. word(1:7).eq.'A-Const'
     $   .or. word(1:7).eq.'A-const') then 
        iwhat=1
      else if (word(1:7).eq.'G-CONST' .or. word(1:7).eq.'G-Const'
     $   .or. word(1:7).eq.'G-const') then 
        iwhat=2
      else
        print *,'Keyword to identify the type of constraint must be'
        print *,
     $    '''G-Constraint'' (or empty field) or ''A-Constraint''.' 
        CALL PROEND('Error reading MEM file')
      endif
      if (iwhat.eq.1 .and. nrefG.gt.0) then
        print *,
     $   'A-constraints must be listed before the G-constraints'
        CALL PROEND('Error in MEM-input file')
      endif
      if (iwhat.eq.2.and.nrefG.gt.0) then
        CALL PROEND('Duplicate G-constraint list given')
      else if (iwhat.eq.1.and.nrefA.gt.0) then
        CALL PROEND('Duplicate A-constraint list given')
      endif
      if (iwhat.eq.2) then
        nrefG=n
        CALL READGOBS(mref,nref,nrefF,nrefA,nrefG,
     $     IDX,GOBS,SIGG,NGRP,PMUL,PMULS)
      else if (iwhat.eq.1) then
        nrefA=n
        CALL READAOBS(mref,nref,nrefA,IDX,AOBS,SIGA,COSPSI,SINPSI)
      endif
      GOTO 25
C
   30 CLOSE(10)
      if ((nrefG.eq.0).and.(nrefA.eq.0)) igsw=1
      write(*,'(i5,a)') nrefF, ' reflections  Fobs found'
      write(*,'(i5,a)') nrefA, ' reflections  Aobs found'
      write(*,'(i5,a)') nrefG, ' refl.-groups Gobs found'
      write(*,'(i5,a)') nref , ' reflections alltogether '
      return
      end
C     -------------------
      SUBROUTINE READGOBS(mref,nref,nrefF,nrefA,nrefG,
     $    IDX,GOBS,SIGG,NGRP,PMUL,PMULS)
C     *** PMUL is the multiplicity of one reflection of the group
C     *** PMULS is the sum of all multiplicities of one group
C     *** GOBS is |G| is the sqrt( sum mult_i*|F_i|^2/(sum mult_i) )
C
      integer mref,nref,nrefF,nrefA,nrefG,iref,i,j,igrp,mul,NGRP(mref/2)
      integer IDX(3,mref)
      real*8 GOBS(mref/2),SIGG(mref/2),
     $     PMUL(mref/2),PMULS(mref/2)
C
      if (nrefG.gt.mref/2) then
        print *,'Maximal number of groups of reflections:',mref/2
        print *,'Given number                           :',nrefG
        CALL PROEND('Too many Gobs given.')
      endif
      if (nrefG.le.0) CALL PROEND('No Gobs found.')
      DO 20 iref=1,nrefG
        READ(10,*) NGRP(iref)
        PMULS(iref)=0.0
        DO 25 igrp=1,NGRP(iref)
          nref=nref+1
          IF (nref.gt.mref) CALL PROEND(
     $      'DIMENSION OVER FOR FCAL - TOO MANY REFLECTIONS')
          IF (igrp.EQ.1) THEN
            READ(10,*) (IDX(I,nref),I=1,3),GOBS(iref),SIGG(iref),mul
C           print *,(IDX(I,nref),I=1,3),GOBS(iref),SIGG(iref),mul
            SIGG(iref)=SIGG(iref)**2
          ELSE
            READ(10,*) (IDX(I,nref),I=1,3),mul
C           print *, (IDX(I,nref),I=1,3),mul
          END IF
	  if (mul.eq.0) then
C           print *,(IDX(I,nref),I=1,3),mul
            CALL proend(
     $      'Multiplicity of Gobs reflection is zero')
          endif
          if (IDX(1,nref).eq.0 .and. IDX(2,nref).eq.0
     $      .and.IDX(3,nref).eq.0 ) CALL PROEND(
     $      'Indices hkl are zero in Gobs') 
          PMUL(nref-(nrefF+nrefA))=DFLOAT(mul)
          PMULS(iref)=PMULS(iref)+DFLOAT(mul)
   25   CONTINUE
   20 CONTINUE
C
      return
      end
C     -------------------
      SUBROUTINE READAOBS(mref,nref,nrefA,IDX,AOBS,SIGA,COSPSI,SINPSI)
C
      integer mref,nref,nrefA,iref,i,j,IDX(3,mref)
      real*8 AOBS(2,mref),SIGA(2,mref),COSPSI(mref),SINPSI(mref),
     $       psi,pi
C
      pi=3.141592654
      iref=0
      if (nrefA.le.0) CALL PROEND('No Aobs found')
      DO 10 i=1,nrefA
        psi=0
        read(10,*,ERR=30) (IDX(j,nref+1),j=1,3),
     $    AOBS(1,iref+1),AOBS(2,iref+1),
     $    SIGA(1,iref+1),SIGA(2,iref+1),psi
        if (nref.ge.mref) then
          print *,'Too many reflections -- some were ignored'
          goto 30
        endif
        iref=iref+1 
        nref=nref+1
        SIGA(1,iref)=SIGA(1,iref)**2
        SIGA(2,iref)=SIGA(2,iref)**2
        COSPSI(iref)=cos(psi/180*pi)
        SINPSI(iref)=sin(psi/180*pi)
C        write(*,999) (aobs(k,iref),k=1,2),(siga(k,iref),k=1,2),psi
999     format(2(f12.2,1x),3x,'sig=',2(f10.3,1x),3x,'psi_sig=',f12.2)
10    continue
30    nrefA=iref   
      return
      end
