C====================================================================
C
C     transpose.f  liest alte MaxEnt-Dichtedateien ein und speichert
C     		   sie mit transponierter Rotationsmatrix ab. 
C		   Frueher wurde die Rotationsmatrix abgespeichert mit:
C                    write(20,1100) ((IROT(i,j,isym),i=1,3),j=1,3),
C		   jetzt ist es das Format:
C		     write(20,1100) ((IROT(i,j,isym),j=1,3),i=1,3),
C		   d.h.  die Rotationsmatrix und der Translationsvektor
C		   eines Symmetrieoperators stehen jetzt so am Ende
C		   der Dichte-Datei:
C			  (r11 r12 r13) t1
C		          (r21 r22 r23) t2
C		          (r31 r32 r33) t3
C
C     K. Burger   Mai 1996, Univ. Tuebingen.
C
C====================================================================
      PROGRAM transpos
C
      IMPLICIT none
      integer*4 nasym,masym
      integer mpix
C
C     Parameters: 
C       masym = max. number of pixels in asymmetric unit of cell
C       mpix = max. number of pixels along one cell axis (resolution)
C
      PARAMETER (masym=135000,mpix=125)
C       CELL: cell parameters a,b,c,alpha,beta,gamma
C       NPIX: no. of pixels in cell-axes directions
C       RDEN: array with density in asymmetric unit
C       IASYM: corresponding ix,iy,iz coordinates of pixels in a.u.
C       IROT,RTRS,RSUB: symmetry operators
C
C     *** variables connected with the scattering-density values in the 
C     asymmetric unit of the cell and in the whole reconstructed cell:
      real*8 RDEN(MASYM)
C
C     *** variables connected with the coordinates of the pixels:
      integer NPIX(3),IASYM(3,MASYM)
C
C     *** cell-constants and others (from PRECAL subroutine):
      real*8 CELL(6)
C
C     *** variables connected with symmetry operators:
      integer IROT(3,3,48),nsym,ncen,nsub,nsg,nset
      real*8 RTRS(3,48),RSUB(3,4)
      integer IHELP(3,3,48)
C
C     *** other variables:
      CHARACTER title*200,filden*50,aspg*10
      integer i,j,isym,nyes
C
 1000 FORMAT((' '))
 1090 format(2(3(I3,1X),F18.6,1x))
 2000 FORMAT(BN,I3)
 2010 FORMAT(A50)
 3010 FORMAT(/' ==============================================='
     $      //'   transpos.f  --  transposing of the symmetry'
     $      //'   operators rotation matrices at the end of old'
     $      //'   MEM density data-files. K. Burger, mai 1996'
     $      //' ==============================================='
     $      //' INPUT DENSITY FILE''S NAME    = ')
 3020 FORMAT( ' ARE YOU SURE ?  YES(0)/NO(1) = ')
C
C
C*** READ DENSITY FILE & RECONSTRUCT DENSITY ***
C
   10 WRITE(*,1000)
      WRITE(*,3010)
      READ(*,2010,ERR=10) filden
      WRITE(*,3020)
      READ(*,2000,ERR=10) nyes
      IF(nyes.NE.0) GOTO 10
C
C     *** read crystal parameters, asymmetric unit and symmetry operators
      CALL READD2(filden,masym,nasym,IASYM,RDEN,
     $   title,CELL,mpix,npix,nsym,ncen,nsub,IROT,RTRS,RSUB)

      print *,title
      write(*,*) '   Points in asym. unit= ',nasym
      print *
      print *,'Symmetry operators before change:'
C     *** display of symmetry info ***
      CALL dspsym(nsg,nset,aspg,nsym,ncen,nsub,
     $  IROT,RTRS,RSUB,CELL)

C     *** transpose rotation matrix:
      do 222 i=1,3
      do 222 j=1,3
      do 222 isym=1,nsym
        IHELP(i,j,isym)=IROT(j,i,isym)
222   continue
      do 223 i=1,3
      do 223 j=1,3
      do 223 isym=1,nsym
        IROT(i,j,isym)=IHELP(i,j,isym)
223   continue
      print *,'Symmetry operators after change:'
C     *** display of symmetry info ***
      CALL dspsym(nsg,nset,aspg,nsym,ncen,nsub,
     $  IROT,RTRS,RSUB,CELL)
 
      print *, 'Ok, saving file ',filden
      CALL SAVEDEN(filden,
     $      masym,nasym,title,IASYM,RDEN,CELL,NPIX
     $      ,nsym,ncen,nsub,IROT,RTRS,RSUB )
      print *,'Finished.'
      END
C -------------------------------------------------------------
C     Read density data, and additional constants and symmetry operations 
C     at the end of the density file 
C     this routine is called by the meco program
C
      SUBROUTINE READD2(filden,masym,nasym,IASYM,RDEN,
     $   title,CELL,mpix,npix,nsym,ncen,nsub,IROT,RTRS,RSUB)

      IMPLICIT INTEGER (I-N)
      IMPLICIT REAL*8 (A-H,O-Z)
      integer*4 masym,nasym,ia
C     *** no of pixels along a,b,c axis; ix,iy,iz of pixels in asymm. unit ***
      dimension NPIX(3),IASYM(3,masym)
C     *** symmetry operators ***
      dimension IROT(3,3,48),RTRS(3,48),RSUB(3,4)
      dimension ITRS(3,48),ISUB(3,4)
C     ***  density array
      dimension RDEN(masym)
C     *** cell constants
      dimension CELL(6)
      character filden*50,title*200
C
      OPEN(20,FILE=filden,FORM='FORMATTED',
     +     STATUS='OLD', ACCESS='SEQUENTIAL')
1080  format(a)
1040  format(i7)
1090  format(2(3(I3,1X),G18.13,1x))

C     *** read title, number of pixels in asymm. unit and density array
      READ(20,1080) title
      READ(20,1040) nasym
      if (nasym.GT.masym) then
	 print *,'maximal number of pixels in asymmetric unit is'
     $     ,masym
	 print *,'number of pixels in density file is',nasym
         CALL PROEND(
     $   'DIMENSION OVER FOR RDEN - TOO MANY POINTS IN ASYM. UNIT')
      endiF
C     READ(20,1090,END=40) ((IASYM(I,ia),I=1,3),RDEN(ia),ia=1,nasym)
      READ(20,*   ,END=40) ((IASYM(I,ia),I=1,3),RDEN(ia),ia=1,nasym)

C     *** read cell constants, symmetry operators and pixel-resolution
      READ(20,'(6F10.5)') (CELL(I),I=1,6)
      READ(20,'(6I5)') (NPIX(I),I=1,3),nsym,ncen,nsub

      if (NPIX(1).GT.mpix.OR.NPIX(2).GT.mpix.OR.NPIX(3).GT.mpix)
     $  then
	write(*,'(a,3i4)') 'pixel resolution is ',(NPIX(i),i=1,3)
	write(*,'(a,3i4)') 'max. pixel resolution is ',mpix
	CALL PROEND('TOO HIGH RESOLUTION ALONG CELL AXES')
      endif

C     READ(20,'(12I5)')
      READ(20,*)
     $    (((IROT(I,J,K),j=1,3),i=1,3),(ITRS(I,K),I=1,3),K=1,nsym)
      READ(20,'(3I5)') ((ISUB(I,K),I=1,3),K=1,nsub)
C
C     *** ISUB() and ITRS() are in pixel units - bring to relative cell-coord.
      do 10 is=1,nsym
      do 10 i=1,3
10       RTRS(i,is)=ITRS(i,is)/(1.0*NPIX(i))
      do 11 is=1,nsub
      do 11 i=1,3
11       RSUB(i,is)=ISUB(i,is)/(1.0*NPIX(i))

      close(20)
      return

   40 CLOSE(20)
      print *,'Number of density pixels in asym. unit should be',
     $   nasym
      print *,'Number of density pixels in asym. unit is ', ia
      CALL PROEND('Error reading density data.')
      return
      end
C
C -------------------------------------------------------------
C     save density of RDEN() array, together with symmetry information 
C
      SUBROUTINE SAVEDEN(filden,
     $   masym,nasym,title,IASYM,RDEN,CELL,NPIX
     $   ,nsym,ncen,nsub,IROT,RTRS,RSUB)
      IMPLICIT none
      integer*4 masym,nasym,ia
      real*8 RDEN(masym),CELL(6),RTRS(3,48),RSUB(3,4)
      integer IASYM(3,masym),NPIX(3),IROT(3,3,48)
      integer nsym,ncen,nsub,i,j,isym
      character title*200,filden*50
C
      OPEN(20,FILE=FILDEN,FORM='FORMATTED',
     $     STATUS='UNKNOWN', ACCESS='SEQUENTIAL')
1010  format(6F10.5)
1020  format(6i5)
1040  format(i7)
1070  format(3i5)
1080  format(a200)
1090  format(2(3(I3,1X),F20.7,1x))
C   1100  format(12i5)       ! old symmetry operator format
1100  format(3(3i4,2x),2x,3(2x,i3)) 
      write(20,1080) title
      write(20,1040) nasym
C     write(20,1090) ((IASYM(i,ia),i=1,3),RDEN(ia),ia=1,nasym)
      do 10 ia=1,nasym
10      write(20,*   ) (IASYM(i,ia),i=1,3),sngl(RDEN(ia))
      write(20,1010) (CELL(i),i=1,6)
      write(20,1020) (NPIX(i),i=1,3),nsym,ncen,nsub
      do 20 isym=1,nsym
        write(20,1100) ((IROT(i,j,isym),j=1,3),i=1,3),
     $                 (NINT(NPIX(i)*RTRS(i,isym)),i=1,3)
20    continue
      write(20,1070) 
     $  ((NINT(NPIX(i)*RSUB(i,isym)),i=1,3),isym=1,nsub)
      CLOSE(20)
      return
      end
C
C -------------------------------------------------------------
C     display symmetry operators
C
      SUBROUTINE dspsym(nsg,nset,aspg,nsym,ncen,nsub,
     $  IROT,RTRS,RSUB,CELL)
      IMPLICIT none
      real*8 RTRS(3,48),RSUB(3,4),CELL(6)
      integer IROT(3,3,48)
      integer nsg,nset,nsym,ncen,nsub,i,j,isym
      character aspg*10
C
      print *,'DISPLAY OF SYMMETRY INFORMATION'
      print *,'==============================='
      print *,'Spacegroup ',aspg
      write(*,'(a,i4,a,i2)') ' Number',nsg,' setting',nset
      write(*,'(a,i4)') ' Number of symm. operators given:',nsym
      write(*,'(a,i2)') ' Number of sub-translations:',nsub
      if (ncen.eq.0) print *,'No center of inversion present'
      if (ncen.eq.1) print *,'Center of inversion present'
      print *,'Cell-parameters a,b,c,alpha,beta,gamma:'
      write(*,'(6(f10.5,1x))') (CELL(i),i=1,6)
      print *,'IROT(3,3,nsym), RTRS(3,nsym):'
      write(*,'(3(3i4,2x),2x,3f5.3)') 
     $  (((IROT(i,j,isym),j=1,3),i=1,3),
     $  (RTRS(i,isym),i=1,3), isym=1,nsym)
      print  *,'RSUB(3,nsym):'
      write(*,'(3f5.3)') ((RSUB(i,isym),i=1,3),isym=1,nsub)
      print *,'Enter <RETURN> to continue'
      read(*,'(BN,i1)') i
C
      RETURN
      END
C -------------------------------------------------------------
C     TERMINATE PROGRAM BECAUSE ERROR OCCURED.
C
      SUBROUTINE PROEND(MESS)
C
      CHARACTER*(*) MESS
C
 3000 FORMAT(///' !!! ERROR !!! : ',A)
C
      WRITE(6,3000) MESS
      WRITE(30,3000) MESS
      CLOSE(30)
C
      STOP
      END
